#!/usr/bin/perl

# Expand an L-system, given an axiom and rules.  Not that robust, but should
# work on most L-systems
# 
# Copyright (C) 2002 Ron Coscorrosa, released under the GNU GPL
#
# Usage:  perl expand.pl level axiom rule1 [rule2 [, rule3 [, .. rule N]]]


use strict;

my $depth = @ARGV ? shift(@ARGV) : "2";
my $axiom = lc(shift(@ARGV)) || lc('f');
my @rules = @ARGV ? @ARGV : ('f=f-f++f-f');

my %rules;

print "Axiom: $axiom\n";

for (@rules)
{
  my ($name, $rule) = map {lc} split /=/;
  $rules{$name} = $rule;
  print " Rule: $name = $rule\n";
}

print "Depth: $depth\n";

my $i;
for $i(0..$depth-1)
{
   my $new_axiom;
   print "level $i:\n$axiom\n\n";
   for (split (//, $axiom))
   {
	$new_axiom .= (exists($rules{$_})) ? $rules{$_} : $_;
   }
   $axiom = $new_axiom;
}

print "\nResulting string:\n";
print $axiom, "\n";

print "\nWithout irrelevant letters:\n";

for (split (//, $axiom)) { print $_ if (/[0-9\@mgfd+-\[\]!]/); }

print "\n\nWithout !'s:\n";

  my $mode = 'norm';
  my $map = {'norm' => {'+' => '+', '-' => '-'},
             'rev' => {'+' => '-', '-' => '+'}};

  my $new;

  for (split (//, $axiom))
  {
     if (/\!/)
     {
	     $mode = ($mode eq 'norm') ? 'rev' : 'norm';
     }
     elsif (/[mgfd+-\[\]]/)
     {
	     $new .= (/[+-]/) ? $map->{$mode}->{$_} : $_;
     }
  }

print $new, "\n";

